@@ -26,6 +26,425 @@ set_filename() {
 }
 
 
+
+query_order=""
+
+separator_single="--------------------------------------------------------------------------------"
+separator_double="================================================================================"
+
+# Create temporary files to store skip and error summary entries.
+SKIP_TMP=$(mktemp /tmp/nvidia_bug_report_skip.XXXXXX)
+ERROR_TMP=$(mktemp /tmp/nvidia_bug_report_error.XXXXXX)
+
+store_result() {
+    description="$1"  # e.g., "NVIDIA GPU Details"
+    details="$2"      # Details to display
+
+    # Create a sanitized version of the description for variable names
+    desc_sanitized=`echo "$description" | tr -c '[:alnum:]_' '_'` 
+
+    # We'll store "SANITIZED|ORIGINAL" on one line (single-char delimiter).
+    line_item="$desc_sanitized|$description"
+
+    query_order="$query_order
+$line_item"
+    eval "results_details_${desc_sanitized}=\"\$details\""
+}
+
+store_skip() {
+    description="$1"
+    details="$2"
+    description=$(echo "$description" | tr '\n' ' ')
+    details=$(echo "$details"       | tr '\n' ' ')
+    echo "$description|$details" >> "$SKIP_TMP"
+}
+
+store_error() {
+    description="$1"
+    details="$2"
+    # Trim any leading/trailing whitespace from resolution.
+    resolution=$(echo "$3" | sed 's/^[[:space:]]*//;s/[[:space:]]*$//')
+    description=$(echo "$description" | tr '\n' ' ')
+    details=$(echo "$details"       | tr '\n' ' ')
+    resolution=$(echo "$resolution" | tr '\n' ' ')
+    printf "%s|%s|%s\n" "$description" "$details" "$resolution" >> "$ERROR_TMP"
+}
+
+report_skip() {
+    description="$1"
+    details="$2"
+    {
+       echo "____________________________________________"
+       echo ""
+       echo "Skipping: $description"
+       echo "$details"
+       echo ""
+    } | $GZIP_CMD >> "$LOG_FILENAME"
+    store_skip "$description" "$details"
+}
+
+report_and_store_error() {
+    description="$1"
+    details="$2"
+    exit_block="$3"  # Optional parameter to indicate whether to exit the block
+    resolution="$4"
+    if [ -z "$resolution" ]; then
+        resolution="Contact NVIDIA for support"
+    fi
+    
+    store_error "$description" "$details" "$resolution"
+
+    {
+       echo "____________________________________________"
+       echo ""
+       echo "Error: $description"
+       echo "$details"
+       echo "Resolution: $resolution"
+       echo ""
+    } | $GZIP_CMD >> "$LOG_FILENAME"
+
+    if [ "$exit_block" = "1" ]; then
+        echo "Exiting current block due to critical error: $description" | $GZIP_CMD >> "$LOG_FILENAME"
+        exit 1  # Exit the subshell to stop the current block
+    fi
+}
+
+print_skips_table() {
+    col1_header="Skipped Component"
+    col1_width=35
+
+    printf "%-${col1_width}s | Details\n" "$col1_header"
+    echo "$separator_double"
+
+    DETAIL_WIDTH=60
+    while IFS='|' read -r description details; do
+        [ -z "$description" ] && continue
+        wrapped=$(printf '%s\n' "$details" | fold -s -w $DETAIL_WIDTH)
+        first_line=$(printf '%s\n' "$wrapped" | sed -n '1p')
+        printf "%-${col1_width}s | %s\n" "$description" "$first_line"
+        rest_lines=$(printf '%s\n' "$wrapped" | sed '1d')
+        if [ -n "$rest_lines" ]; then
+            echo "$rest_lines" | while IFS= read -r detail_line; do
+                printf "%-${col1_width}s | %s\n" "" "$detail_line"
+            done
+        fi
+        echo "$separator_single"
+    done < "$SKIP_TMP"
+}
+
+print_errors_table() {
+    col1_width=35
+    col2_width=60
+    col3_width=20
+    header=$(printf "%-${col1_width}s | %-${col2_width}s | %-${col3_width}s" "Error Component" "Details" "Resolution")
+    sep_line=$(printf '%*s' $(expr $col1_width + $col2_width + $col3_width + 6) '' | tr ' ' '=')
+    echo "$header"
+    echo "$sep_line"
+    while IFS='|' read -r comp details resolution; do
+        [ -z "$comp" ] && continue
+
+        comp_wrapped=$(printf '%s\n' "$comp" | fold -s -w $col1_width)
+        details_wrapped=$(printf '%s\n' "$details" | fold -s -w $col2_width)
+        resolution_wrapped=$(printf '%s\n' "$resolution" | fold -s -w $col3_width)
+
+        comp_lines=$(printf '%s\n' "$comp_wrapped")
+        details_lines=$(printf '%s\n' "$details_wrapped")
+        resolution_lines=$(printf '%s\n' "$resolution_wrapped")
+        comp_count=$(printf '%s\n' "$comp_lines" | wc -l | sed 's/ //g')
+        details_count=$(printf '%s\n' "$details_lines" | wc -l | sed 's/ //g')
+        resolution_count=$(printf '%s\n' "$resolution_lines" | wc -l | sed 's/ //g')
+        max_lines=$(printf "%s\n" "$comp_count" "$details_count" "$resolution_count" | sort -nr | head -n 1)
+        i=1
+        while [ $i -le $max_lines ]; do
+            comp_line=$(printf '%s\n' "$comp_lines" | sed -n "${i}p")
+            details_line=$(printf '%s\n' "$details_lines" | sed -n "${i}p")
+            resolution_line=$(printf '%s\n' "$resolution_lines" | sed -n "${i}p")
+            printf "%-${col1_width}s | %-${col2_width}s | %-${col3_width}s\n" "$comp_line" "$details_line" "$resolution_line"
+            i=$(expr $i + 1)
+        done
+        sep_line_single=$(printf '%*s' $(expr $col1_width + $col2_width + $col3_width + 6) '' | tr ' ' '-')
+        echo "$sep_line_single"
+    done < "$ERROR_TMP"
+}
+
+print_versions_table() {
+    order_list="$query_order"
+    col1_header="Component"
+
+    col1_width=35
+
+    # Print header
+    printf "%-${col1_width}s | Details\n" "$col1_header"
+    echo "$separator_double"
+
+    echo "$order_list" | while IFS= read -r line
+    do
+        # Skip blank lines
+        [ -z "$line" ] && continue
+
+        # Extract sanitized + original description
+        sanitized="$(printf '%s' "$line" | cut -f1 -d'|')"
+        original="$(printf '%s' "$line" | cut -f2- -d'|')"
+
+        # Retrieve details from dynamic variables
+        eval details="\$results_details_${sanitized}"
+
+        # If details is empty, print "None" in the details column
+        if [ -z "$details" ]; then
+            details="None"
+        fi
+
+        # Split details into first line + rest
+        first_line="$(printf '%s\n' "$details" | sed -n '1p')"
+        printf "%-${col1_width}s | %s\n" "$original" "$first_line"
+
+        rest_lines="$(printf '%s\n' "$details" | sed '1d')"
+        if [ -n "$rest_lines" ]; then
+            echo "$rest_lines" | while IFS= read -r detail_line
+            do
+                printf "%-${col1_width}s | %s\n" "" "$detail_line"
+            done
+        fi
+
+        echo "$separator_single"
+    done
+}
+
+parse_ibv_devinfo() {
+    dev="$1"
+    shift
+    requested_fields="$*"
+
+    devinfo="$(ibv_devinfo -d "$dev" 2>/dev/null)"
+    hca_id="$(echo "$devinfo" | grep -m1 '^hca_id:' | awk '{print $2}')"
+    fw_ver="$(echo "$devinfo" | grep -m1 'fw_ver:' | awk '{print $2}')"
+    link_layer="$(echo "$devinfo" | grep -m1 'link_layer:' | awk '{print $2}')"
+    state="$(echo "$devinfo" | grep -m1 'state:' | sed -E 's/.*state:\s+([^ ]+).*/\1/')"
+
+    output=""
+    for field in $requested_fields
+    do
+        case "$field" in
+            device_name)
+                if [ "$dev" != "$hca_id" ]; then
+                    output="$output""Device Name : $hca_id
+"
+                fi
+                ;;
+            fw_ver)
+                output="$output""Firmware Ver: $fw_ver
+"
+                ;;
+            link_layer)
+                output="$output""Link Layer  : $link_layer
+"
+                ;;
+            state)
+                output="$output""Port State  : $state
+"
+                ;;
+        esac
+    done
+
+    printf "%s" "$output"
+}
+
+run_query_ibv_devinfo() {
+    description="$1"  # e.g., "InfiniBand Firmware Versions"
+
+    # Is ibv_devinfo present?
+    if ! command -v ibv_devinfo >/dev/null 2>&1; then
+        store_result "$description" "None"
+        return
+    fi
+
+    # Gather devices
+    devices="$(ibv_devinfo -l 2>/dev/null | sed -n 's/^[[:space:]]*\(mlx5_[0-9]*\)$/\1/p')"
+
+    # Parse each device
+    desired_fields="device_name fw_ver link_layer state"
+    aggregated_output=""
+
+    for dev in $devices
+    do
+        dev_output="$(parse_ibv_devinfo "$dev" "$desired_fields")"
+        aggregated_output="$aggregated_output""Device      : $dev
+$dev_output
+
+"
+    done
+
+    # Store into our table
+    store_result "$description" "$aggregated_output"
+}
+
+check_command() {
+    cmd="$1"         # Command to check
+    description="$2" # Description for the table entry
+
+    # Check if the command exists
+    if ! command -v "$cmd" >/dev/null 2>&1; then
+        # If the command is not found, log it to the table
+        store_result "$description" "$cmd command not found"
+        return 1
+    fi
+    return 0
+}
+
+run_query_mlxlink() {
+    description="$1"  # e.g., "Mellanox Link Firmware Versions"
+    query="$2"        # Query string to match in 'mst status'
+
+    # Check if required commands exist
+    if ! check_command "mst" "$description"; then return; fi
+    if ! check_command "mlxlink" "$description"; then return; fi
+
+    # Run mst status and capture output
+    mst_output=$(sudo mst status -v 2>&1)
+    if [ $? -ne 0 ]; then
+        store_result "$description" "Error running 'mst status': $mst_output"
+        return
+    fi
+
+    # Initialize result aggregation
+    aggregated_output=""
+    while IFS= read -r tmp_line; do
+        echo "$tmp_line" | grep "$query" | grep "mlx5_0" >/dev/null 2>&1
+        if [ $? -eq 0 ]; then
+            # Extract device name
+            devName=$(echo "$tmp_line" | awk -F" " '{print $1}')
+            mlxlink_output=$(sudo mlxlink -d "$devName" 2>&1)
+
+            if [ $? -ne 0 ]; then
+                aggregated_output="$aggregated_output""Device: $devName
+Error: $(echo "$mlxlink_output" | head -n 1)
+"
+                continue
+            fi
+
+            aggregated_output="$aggregated_output""
+""
+""Device: $devName
+$mlxlink_output
+
+
+"
+        fi
+    done <<EOF
+$(echo "$mst_output" | grep "$query" | grep "mlx5_0")
+EOF
+
+    # Store the result
+    if [ -z "$aggregated_output" ]; then
+        store_result "$description" "No devices matching query '$query'"
+    else
+        store_result "$description" "$aggregated_output"
+    fi
+}
+
+run_query() {
+    cmd="$1"
+    args="$2"
+    description="$3"
+
+    # If command is not found
+    if ! command -v "$cmd" >/dev/null 2>&1; then
+        store_result "$description" "None"
+        return
+    fi
+
+    # Capture all stdout+stderr
+    output=`$cmd $args 2>&1`
+    ret_code=$?
+
+    if [ "$description" = "OS Details" ] && [ "$ret_code" -eq 0 ]; then
+        os_details=""
+        if command -v lsb_release >/dev/null 2>&1; then
+            os_details="$os_details""Distribution : $(lsb_release -d -s)
+"
+        elif [ -f /etc/os-release ]; then
+            distline="$(grep '^PRETTY_NAME=' /etc/os-release | cut -d= -f2- | tr -d '"')"
+            if [ -n "$distline" ]; then
+                os_details="$os_details""Distribution : $distline
+"
+            else
+                os_details="$os_details""Distribution : Unknown
+"
+            fi
+        else
+            os_details="$os_details""Distribution : Unknown
+"
+        fi
+
+        os_details="$os_details""Kernel       : $(uname -r)
+"
+        os_details="$os_details""Hostname     : $(hostname)
+"
+        os_details="$os_details""Architecture : $(uname -m)
+"
+        os_details="$os_details""Uptime       : $(uptime -p 2>/dev/null || true)
+"
+
+        output="$os_details"
+    fi
+
+
+    # Mark success or failure
+    if [ "$ret_code" -eq 0 ]; then
+        store_result "$description" "$output"
+    else
+        first_line="$(echo "$output" | head -n 1)"
+        store_result "$description" "Failed: $first_line"
+    fi
+}
+
+run_query_file() {
+    file_path="$1"
+    pattern="$2"
+    description="$3"
+
+    # Check if file is readable
+    if [ ! -r "$file_path" ]; then
+        store_result "$description" "None"
+        return
+    fi
+
+    # Grab matching lines
+    filtered_output=`grep "$pattern" "$file_path" 2>/dev/null`
+    ret_code=$?
+
+    if [ $ret_code -eq 0 ]; then
+        if [ -z "$filtered_output" ]; then
+            store_result "$description" "None"
+        else
+            store_result "$description" "$filtered_output"
+        fi
+    else
+        store_result "$description" "None"
+    fi
+}
+
+fetch_software_firmware_versions() {
+    run_query "vulkaninfo"     "--version" "Vulkan Info"
+    run_query "nvidia-smi"     "--version" "NVIDIA SMI"
+    run_query "nvidia-settings" "--version" "NVIDIA Settings"
+    run_query "nv-fabricmanager" "--version" "NVIDIA Fabric Manager"
+    run_query "opensm"         "--version" "NVIDIA Subnet Manager"
+    run_query "mlxlink"        "--version" "Mellanox Link"
+    run_query "ibstat"         "--version" "InfiniBand Status"
+    run_query "ibnetdiscover"  "--version" "InfiniBand Network Discovery"
+    run_query_file "/var/log/dmesg" "MSE\|uCode\|NETIR" "NVIDIA MSE/NETIR Versions"
+
+    run_query "nvidia-smi"     "--query-gpu=name,driver_version,memory.total,vbios_version,pci.bus_id,serial --format=csv,noheader" \
+                               "NVIDIA GPU Details"
+    run_query_mlxlink "NVIDIA Switch Details" "Quantum"
+    run_query_ibv_devinfo "NVIDIA NIC Details"
+
+    run_query "lsb_release" "-d" "OS Details"
+
+    print_versions_table
+}
+
 module_names="nvidia nvidia_drm nvidia_modeset nvidia_uvm nvidia_peermem"
 
 usage_bug_report_message() {
@@ -44,25 +463,72 @@ usage() {
     echo "$(basename $0): NVIDIA Linux Graphics Driver bug reporting shell script."
     echo ""
     usage_bug_report_message
+    echo "Usage:"
+    echo "  $(basename $0) [OPTIONS]"
     echo ""
-    echo "$(basename $0) [OPTION]..."
-    echo "    -h / --help"
-    echo "        Print this help output and exit."
-    echo "    --output-file <file>"
-    echo "        Write output to <file>. If gzip is available, the output file"
-    echo "        will be automatically compressed, and \".gz\" will be appended"
-    echo "        to the filename. Default: write to nvidia-bug-report.log(.gz)."
-    echo "    --safe-mode"
-    echo "        Disable some parts of the script that may hang the system."
-    echo "    --extra-system-data"
-    echo "        Enable additional data collection that may aid in the analysis"
-    echo "        of certain classes of bugs. If running the script without the"
-    echo "        --safe-mode option hangs the system, consider using this"
-    echo "        option to help identify stuck kernel software."
+    echo "Options:"
+    echo "  -h, --help"
+    echo "    Print this help message and exit."
+    echo ""
+    echo "  --output-file <file>"
+    echo "    Write output to <file>. If 'gzip' is available, the file will"
+    echo "    automatically be compressed to <file>.gz."
+    echo "    Default: nvidia-bug-report.log(.gz)."
+    echo ""
+    echo "  --safe-mode"
+    echo "    Disable certain queries that might hang the system. Useful if you"
+    echo "    experience freezes, high CPU usage, or suspect problematic kernel modules."
+    echo ""
+    echo "  --extra-system-data"
+    echo "    Gather additional system information and CPU backtraces for deeper"
+    echo "    debugging. Consider using this if the script hangs without --safe-mode"
+    echo "    or you need more extensive kernel data."
+    echo ""
+    #VGX_SPECIFIC_CODE_START
+    if [ -n "$IS_XEN_HYPERVISOR" ]; then
+        echo "  --domain-name \"<domain_name>\""
+        echo "    Generate a bug report for the specified Xen virtual machine domain."
+        echo "    Only valid if running within Xen dom0."
+        echo ""
+    fi
+    #VGX_SPECIFIC_CODE_END
+    echo "  -v, --version"
+    echo "    Show the current version of nvidia-bug-report.sh and exit."
+    echo "  -V, --versions"
+    echo "    Show the current versions of NVIDIA software and firmware on the node."
+    echo ""
+    echo "Description:"
+    echo "  This script collects logs and configuration details—covering kernel"
+    echo "  messages, GPU driver info, Xorg logs, hypervisor data (if applicable),"
+    echo "  and more—to help diagnose NVIDIA driver or GPU-related issues. Run it"
+    echo "  as root or with sudo for the most complete data."
+    echo ""
+    echo "  If certain tools (e.g., 'nvidia-smi', 'lspci', 'mstflint') are missing,"
+    echo "  the script will note \"Skipping...\" in the final report but continue"
+    echo "  collecting other data."
+    echo ""
+    echo "Troubleshooting Tips:"
+    echo "  - If the script hangs, try using '--safe-mode'."
+    echo "  - If you need deeper system info (e.g., CPU backtraces), use '--extra-system-data'."
+    echo "  - Check final '.log' or '.log.gz' for 'Skipping...' lines to see if any"
+    echo "    dependencies were missing or commands failed."
+    echo "  - Ensure you have installed recommended dependencies (see the full documentation"
+    echo "    or 'system_requirements.rst'). Official releases of the NVIDIA driver/firmware"
+    echo "    reduce hangs and incomplete captures."
+    echo ""
+    echo "Examples:"
+    echo "  1) sudo ./nvidia-bug-report.sh"
+    echo "     Creates 'nvidia-bug-report.log.gz' if 'gzip' is installed."
+    echo ""
+    echo "  2) sudo ./nvidia-bug-report.sh --safe-mode"
+    echo "     Skips queries known to cause hangs on certain systems."
+    echo ""
+    echo "  3) sudo ./nvidia-bug-report.sh --extra-system-data --output-file custom_report"
+    echo "     Gathers extended kernel info into 'custom_report.log.gz' (compressed if possible)."
     echo ""
 }
 
-NVIDIA_BUG_REPORT_CHANGE='$Change: 35030023 $'
+NVIDIA_BUG_REPORT_CHANGE='$Change: 35734984 $'
 NVIDIA_BUG_REPORT_VERSION=`echo "$NVIDIA_BUG_REPORT_CHANGE" | tr -c -d "[:digit:]"`
 
 # Set the default filename so that it won't be empty in the usage message
@@ -74,28 +540,43 @@ BUG_REPORT_SAFE_MODE=0
 BUG_REPORT_EXTRA_SYSTEM_DATA=0
 SAVED_FLAGS=$@
 while [ "$1" != "" ]; do
-    case $1 in
-        -o | --output-file )    if [ -z $2 ]; then
-                                    usage
-                                    exit 1
-                                elif [ "$(echo "$2" | cut -c 1)" = "-" ]; then
-                                    echo "Warning: Questionable filename"\
-                                         "\"$2\": possible missing argument?"
-                                fi
-                                BASE_LOG_FILENAME="$2"
-                                # override the default filename
-                                set_filename
-                                shift
-                                ;;
-        --safe-mode )           BUG_REPORT_SAFE_MODE=1
-                                ;;
-        --extra-system-data )   BUG_REPORT_EXTRA_SYSTEM_DATA=1
-                                ;;
-        -h | --help )           usage
-                                exit
-                                ;;
-        * )                     usage
-                                exit 1
+    case "$1" in
+        -o | --output-file )
+            if [ -z "$2" ]; then
+                echo "Error: --output-file requires a filename argument."
+                usage
+                exit 1
+            elif [ "$(echo "$2" | cut -c 1)" = "-" ]; then
+                echo "Warning: Questionable filename \"$2\": possible missing argument?"
+            fi
+            BASE_LOG_FILENAME="$2"
+            # Override the default filename
+            set_filename
+            shift
+            ;;
+        --safe-mode )
+            BUG_REPORT_SAFE_MODE=1
+            ;;
+        --extra-system-data )
+            BUG_REPORT_EXTRA_SYSTEM_DATA=1
+            ;;
+        -h | --help )
+            usage
+            exit 0
+            ;;
+        -v | --version )
+            echo "${NVIDIA_BUG_REPORT_VERSION}"
+            exit 0
+            ;;
+        -V | --versions )
+            fetch_software_firmware_versions
+            exit 0
+            ;;
+        * )
+            echo "Error: Unknown option \"$1\"."
+            usage
+            exit 1
+            ;;
     esac
     shift
 done
@@ -205,7 +686,7 @@ append_binary_file() {
                     echo ""
                 fi
         else
-            echo "Skipping $1 output (base64 not found)"
+            report_skip "$1 output" "base64 not found"
             echo ""
         fi
 
@@ -424,6 +905,11 @@ echo -n "Running $(basename $0)...";
     echo ""
 ) | $GZIP_CMD >> $LOG_FILENAME
 
+# print software firmware versions
+(
+    fetch_software_firmware_versions
+) | $GZIP_CMD >> $LOG_FILENAME
+
 if [ $BUG_REPORT_SAFE_MODE -eq 0 ]; then
     # List the sysfs entries for all NVIDIA device functions
     # This info is useful to debug dynamic power management issues
@@ -483,6 +969,7 @@ append_silent "/etc/release"
 append_silent "/etc/gentoo-release"
 
 
+
 append "/var/log/nvidia-installer.log"
 append_silent "/var/log/nvidia-uninstall.log"
 
@@ -499,7 +986,7 @@ fi
 systemctl=`which systemctl 2> /dev/null | head -n 1`
 
 if [ $? -eq 0 -a -x "$systemctl" ]; then
-    cmd="$systemctl status nvidia-suspend.service nvidia-hibernate.service nvidia-resume.service nvidia-powerd.service"
+    cmd="$systemctl status nvidia-suspend.service nvidia-hibernate.service nvidia-resume.service nvidia-powerd.service nvidia-persistenced.service"
     (
         echo "____________________________________________"
         echo ""
@@ -641,7 +1128,7 @@ append_file_or_dir_silent /usr/share/nvidia/nvidia-application-profiles-*-rc
         ldd $glxinfo 2> /dev/null
         echo ""
     else
-        echo "Skipping ldd output (glxinfo not found)"
+        report_skip "ldd output" "glxinfo not found"
         echo ""
     fi
 ) | $GZIP_CMD >> $LOG_FILENAME
@@ -700,8 +1187,46 @@ append_file_or_dir_silent /usr/share/nvidia/nvidia-application-profiles-*-rc
         echo ""
         $lspci -nnDvvvxxxx 2> /dev/null
     else
-        echo "Skipping lspci output (lspci not found)"
-        echo ""
+        hexdump=`which hexdump 2> /dev/null | head -n 1`
+        if [ $? -eq 0 -a -x "$hexdump" ]; then
+            # Capture hex dump of whole configuration space using sysfs file
+            echo "PCI devices configuration space dump using sysfs"
+            echo ""
+
+            # Define the base path for PCI devices
+            PCI_BASE_PATH="/sys/bus/pci/devices"
+
+            # Loop through each PCI device in the directory
+            for PCI_BDF in "$PCI_BASE_PATH"/*; do
+                # Extract device ID (BDF format)
+                PCI_ID=$(basename "$PCI_BDF")
+
+                # Define the config file path
+                PCI_CONFIG="$PCI_BDF/config"
+
+                # Check if the config file exists
+                if [ ! -f "$PCI_CONFIG" ]; then
+                    continue
+                fi
+
+                # Retrieve PCI device information from sysfs and
+                # remove "0x" prefix if present
+                VENDOR_ID=$(sed s/^0x// "$PCI_BDF"/vendor)
+                DEVICE_ID=$(sed s/^0x// "$PCI_BDF"/device)
+                REVISION_ID=$(sed s/^0x// "$PCI_BDF"/revision)
+                CLASS_ID=$(sed s/^0x// "$PCI_BDF"/class)
+
+                # Print PCI device information (mimicking `lspci`)
+                echo
+                echo "$PCI_ID $CLASS_ID: Vendor $VENDOR_ID Device $DEVICE_ID (rev $REVISION_ID)"
+
+                # Read the config file using 'hexdump' and format output
+                $hexdump -v -e '16/1 "%02x " "\n"' "$PCI_CONFIG" | awk '{printf "%02x: %s\n",NR*16-16, $0}'
+            done
+        else
+            report_skip "lspci output" "lspci or hexdump not found"
+            echo ""
+        fi
     fi
 ) | $GZIP_CMD >> $LOG_FILENAME
 
@@ -765,7 +1290,7 @@ append /proc/sys/vm/compaction_proactiveness
         $lsusb 2> /dev/null
         echo ""
     else
-        echo "Skipping lsusb output (lsusb not found)"
+        report_skip "lsusb output" "lsusb not found"
         echo ""
     fi
 ) | $GZIP_CMD >> $LOG_FILENAME
@@ -784,7 +1309,7 @@ append /proc/sys/vm/compaction_proactiveness
         $dmidecode 2> /dev/null
         echo ""
     else
-        echo "Skipping dmidecode output (dmidecode not found)"
+        report_skip "dmidecode output" "dmidecode not found"
         echo ""
     fi
 ) | $GZIP_CMD >> $LOG_FILENAME
@@ -805,7 +1330,7 @@ append /proc/sys/vm/compaction_proactiveness
             echo ""
         done
     else
-        echo "Skipping modinfo output (modinfo not found)"
+        report_skip "modinfo output" "modinfo not found"
         echo ""
     fi
 ) | $GZIP_CMD >> $LOG_FILENAME
@@ -977,7 +1502,7 @@ if [ $BUG_REPORT_SAFE_MODE -eq 0 ]; then
                  echo "xrandr could not connect to an X server"
             fi
         else
-            echo "Skipping xrandr output (xrandr not found)"
+            report_skip "xrandr output" "xrandr not found"
         fi
     ) | $GZIP_CMD >> $LOG_FILENAME
 
@@ -1105,7 +1630,7 @@ done
             touch $NVDD_TEMP_FILENAME 2> /dev/null
 
             if [ $? -ne 0 ]; then
-                echo "Skipping nvidia-debugdump output (can't create temp file $NVDD_TEMP_FILENAME)"
+                report_skip "nvidia-debugdump output" "Can't create temp file $NVDD_TEMP_FILENAME"
                 echo ""
                 # don't fail here, continue
             else
@@ -1119,11 +1644,11 @@ done
                 rm $NVDD_TEMP_FILENAME 2> /dev/null
             fi
         else
-            echo "Skipping nvidia-debugdump output (base64 not found)"
+            report_skip "nvidia-debugdump output" "base64 not found"
             echo ""
         fi
     else
-        echo "Skipping nvidia-debugdump output (nvidia-debugdump not found)"
+        report_skip "nvidia-debugdump output" "nvidia-debugdump not found"
         echo ""
     fi
 ) | $GZIP_CMD >> $LOG_FILENAME
@@ -1145,7 +1670,7 @@ if [ $BUG_REPORT_SAFE_MODE -eq 0 ]; then
             $vulkaninfo 2> /dev/null
             echo ""
         else
-            echo "Skipping vulkaninfo output (vulkaninfo not found)"
+            report_skip "vulkaninfo output" "vulkaninfo not found"
             echo ""
         fi
     ) | $GZIP_CMD >> $LOG_FILENAME
@@ -1170,8 +1695,9 @@ if [ $BUG_REPORT_SAFE_MODE -eq 0 ]; then
             report_command "$nvidia_smi nvlink --errorcounters"
             report_command "$nvidia_smi nvlink --remotelinkinfo"
             report_command "$nvidia_smi nvlink --status"
+            report_command "$nvidia_smi conf-compute --query-conf-compute"
         else
-            echo "Skipping nvidia-smi output (nvidia-smi not found)"
+            report_skip "nvidia-smi output" "nvidia-smi not found"
             echo ""
         fi
     ) | $GZIP_CMD >> $LOG_FILENAME
@@ -1183,7 +1709,69 @@ if [ $BUG_REPORT_SAFE_MODE -eq 0 ]; then
     fi
 else
     (
-        echo "Skipping nvidia-smi, vulkaninfo etc. due to --safe-mode argument."
+        report_skip "nvidia-smi, vulkaninfo etc. Commands" "Skipping due to --safe-mode-argument"
+        echo ""
+    ) | $GZIP_CMD >> $LOG_FILENAME
+fi
+
+# InfiniBand Support
+if [ $BUG_REPORT_SAFE_MODE -eq 0 ]; then
+    (
+        echo "____________________________________________"
+        echo ""
+
+        # Check if ibstat is available to discover devices
+        ibstat_cmd=`which ibstat 2> /dev/null | head -n 1`
+
+        if [ $? -eq 0 -a -x "$ibstat_cmd" ]; then
+            # Run ibstat without device specification (covers all devices)
+            report_command "$ibstat_cmd"
+
+            # List all InfiniBand devices
+            devices=`$ibstat_cmd -l`
+
+            # Iterate over each device
+            for device in $devices; do
+                echo "Running InfiniBand commands for device: $device"
+                echo ""
+
+                # ibnetdiscover
+                ibnetdiscover_cmd=`which ibnetdiscover 2> /dev/null | head -n 1`
+                if [ $? -eq 0 -a -x "$ibnetdiscover_cmd" ]; then
+                    report_command "$ibnetdiscover_cmd -C $device"
+                else
+                    report_skip "ibnetdiscover output" "ibnetdiscover not found"
+                    echo ""
+                fi
+
+                # ibswitches
+                ibswitches_cmd=`which ibswitches 2> /dev/null | head -n 1`
+                if [ $? -eq 0 -a -x "$ibswitches_cmd" ]; then
+                    report_command "$ibswitches_cmd -C $device"
+                else
+                    report_skip "ibswitches output" "ibswitches not found"
+                    echo ""
+                fi
+
+                # iblinkinfo
+                iblinkinfo_cmd=`which iblinkinfo 2> /dev/null | head -n 1`
+                if [ $? -eq 0 -a -x "$iblinkinfo_cmd" ]; then
+                    report_command "$iblinkinfo_cmd -C $device"
+                else
+                    report_skip "iblinkinfo output" "iblinkinfo not found"
+                    echo ""
+                fi
+
+                echo ""
+            done
+        else
+            report_skip "ibstat output" "ibstat not found"
+            echo ""
+        fi
+    ) | $GZIP_CMD >> $LOG_FILENAME
+else
+    (
+        report_skip "Infiniband Commands" "Skipping due to --safe-mode-argument"
         echo ""
     ) | $GZIP_CMD >> $LOG_FILENAME
 fi
@@ -1282,16 +1870,16 @@ fi
                 # remove the temporary file when complete
                 rm $TEMP_FILENAME 2> /dev/null
             else
-                echo "Skipping acpidump output (can't create data file $TEMP_FILENAME)"
+                report_skip "acpidump output" "Can't create data file $TEMP_FILENAME"
                 echo ""
                 # don't fail here, continue
             fi
         else
-            echo "Skipping acpidump output (base64 not found)"
+            report_skip "acpidump output" "base64 not found"
             echo ""
         fi
     else
-        echo "Skipping acpidump output (acpidump not found)"
+        report_skip "acpidump output" "acpidump not found"
         echo ""
     fi
 ) | $GZIP_CMD >> $LOG_FILENAME
@@ -1308,8 +1896,9 @@ dump_mlx() {
     # print information to stdout (in color).
     # Capture both and remove the color escape characters.
     echo "$1 -d $2 --amber_collect $MLX_CSV > $MLX_INFO 2>&1"
-    "$1" -d "$2" --amber_collect "$MLX_CSV" > "$MLX_INFO" 2>&1
-    if [ $? -eq 0 ]; then
+    mlx_output=`"$1" -d "$2" --amber_collect "$MLX_CSV" > "$MLX_INFO" 2>&1`
+    exit_code=$?
+    if [ $exit_code -eq 0 ]; then
         cat "$MLX_INFO" | sed -Ee 's/\x1b\[[0-9;]+m//g' 2> /dev/null
         cat "$MLX_CSV" 2> /dev/null
         echo "____________________________________________"
@@ -1318,7 +1907,8 @@ dump_mlx() {
         rm -f "$MLX_CSV" 2> /dev/null
         break
     else
-        echo "failed with error $? retrying..."
+        echo "failed with error $exit_code retrying..."
+        echo "$mlx_output"
         cat "$MLX_INFO" | sed -Ee 's/\x1b\[[0-9;]+m//g' 2> /dev/null
         rm -f "$MLX_INFO" 2> /dev/null
         rm -f "$MLX_CSV" 2> /dev/null
@@ -1419,10 +2009,65 @@ dump_mlx() {
                 fi
                 done
             }
+
+            # stop due to resource contention.
+            echo ""
+            report_command "$mst stop"
+            echo "____________________________________________"
+
+        else
+            echo "____________________________________________"
+            echo ""
+            report_skip "mlxlink output" "mlxlink not found"
+            echo ""
         fi
+
+    else
+        echo "____________________________________________"
+        echo ""
+        report_skip "mst output" "mst not found"
+        echo ""
     fi
 ) | $GZIP_CMD >> $LOG_FILENAME
 
+# Collect NVLSM dumps
+(
+    NVLSM_SBIN_PATH=/opt/nvidia/nvlsm/sbin
+
+    echo "____________________________________________"
+    echo ""
+
+    nvlsm_bug_report=`PATH=$PATH:$NVLSM_SBIN_PATH which nvlsm-bug-report.sh 2> /dev/null | head -n 1`
+    if [ $? -eq 0 -a -x "${nvlsm_bug_report}" ]; then
+        echo "Collecting NVLink Subnet Manager (NVLSM) information..."
+        report_command "${nvlsm_bug_report} --stdout"
+    else
+        report_skip "nvlsm-bug-report.sh output" "nvlsm-bug-report.sh not found"
+        echo ""
+    fi
+) | $GZIP_CMD >> $LOG_FILENAME
+
+
+echo " complete."
+echo ""
+
+TMP_LOG=$(mktemp)
+
+{
+  echo ""
+  echo "Summary of Skipped Sections:"
+  echo ""
+  print_skips_table
+  echo ""
+  echo "Summary of Errors:"
+  echo ""
+  print_errors_table
+  echo ""
+} | tee "$TMP_LOG"
+
+cat "$TMP_LOG" | $GZIP_CMD >> "$LOG_FILENAME"
+rm -f "$TMP_LOG"
+
 (
     echo "____________________________________________"
 
@@ -1432,7 +2077,5 @@ dump_mlx() {
     echo "End of NVIDIA bug report log file."
 ) | $GZIP_CMD >> $LOG_FILENAME
 
-# Done
 
-echo " complete."
-echo ""
+rm -f "$SKIP_TMP" "$ERROR_TMP" "$TMP_LOG"
